"""
Functions and routines associated with Enasis Network Chatting Robie.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
"""



from .thread import RobieThread



class RobieClientThread(RobieThread):
    """
    Common methods and routines for Chatting Robie threads.
    """
